@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2014 - 2018 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -28,11 +28,13 @@ from os import linesep
 from random import randint
 from time import sleep
 
-from .. import FIRST, ROUND_ROBIN, RANDOM, POOLING_STRATEGIES, SEQUENCE_TYPES, STRING_TYPES, get_config_parameter
+from .. import FIRST, ROUND_ROBIN, RANDOM, SEQUENCE_TYPES, STRING_TYPES, get_config_parameter
 from .exceptions import LDAPUnknownStrategyError, LDAPServerPoolError, LDAPServerPoolExhaustedError
 from .server import Server
 from ..utils.log import log, log_enabled, ERROR, BASIC, NETWORK
 
+POOLING_STRATEGIES = [FIRST, ROUND_ROBIN, RANDOM]
+
 
 class ServerPoolState(object):
     def __init__(self, server_pool):
@@ -114,7 +116,7 @@ class ServerPoolState(object):
                 if not server[2]:  # server is offline
                     if (isinstance(self.server_pool.exhaust, bool) and self.server_pool.exhaust) or (datetime.now() - server[1]).seconds < self.server_pool.exhaust:  # keeps server offline
                         if log_enabled(NETWORK):
-                            log(NETWORK, 'server <%s> excluded from checking because is offline', server[0])
+                            log(NETWORK, 'server <%s> excluded from checking because it is offline', server[0])
                         continue
                     if log_enabled(NETWORK):
                             log(NETWORK, 'server <%s> reinserted in pool', server[0])
@@ -134,6 +136,7 @@ class ServerPoolState(object):
         raise LDAPServerPoolExhaustedError('no random active server available in server pool after maximum number of tries')
 
     def find_active_server(self, starting):
+        conf_pool_timeout = get_config_parameter('POOLING_LOOP_TIMEOUT')
         counter = self.server_pool.active  # can be True for "forever" or the number of cycles to try
         if starting >= len(self.servers):
             starting = 0
@@ -150,7 +153,7 @@ class ServerPoolState(object):
                     if (isinstance(self.server_pool.exhaust, bool) and self.server_pool.exhaust) or (datetime.now() - self.servers[offset][1]).seconds < self.server_pool.exhaust:  # keeps server offline
                         if log_enabled(NETWORK):
                             if isinstance(self.server_pool.exhaust, bool):
-                                log(NETWORK, 'server <%s> excluded from checking because is  offline', self.servers[offset][0])
+                                log(NETWORK, 'server <%s> excluded from checking because is offline', self.servers[offset][0])
                             else:
                                 log(NETWORK, 'server <%s> excluded from checking because is offline for %d seconds', self.servers[offset][0], (self.server_pool.exhaust - (datetime.now() - self.servers[offset][1]).seconds))
                         continue
@@ -168,8 +171,8 @@ class ServerPoolState(object):
             if not isinstance(self.server_pool.active, bool):
                 counter -= 1
             if log_enabled(NETWORK):
-                log(NETWORK, 'waiting for %d seconds before retrying pool servers cycle', get_config_parameter('POOLING_LOOP_TIMEOUT'))
-            sleep(get_config_parameter('POOLING_LOOP_TIMEOUT'))
+                log(NETWORK, 'waiting for %d seconds before retrying pool servers cycle', conf_pool_timeout)
+            sleep(conf_pool_timeout)
 
         if log_enabled(ERROR):
             log(ERROR, 'no active server available in Server Pool <%s> after maximum number of tries', self)
