@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2014 - 2018 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -25,6 +25,7 @@
 
 from ...extend.operation import ExtendedOperation
 from ...protocol.novell import NmasGetUniversalPasswordRequestValue, NmasGetUniversalPasswordResponseValue, NMAS_LDAP_EXT_VERSION
+from ...utils.dn import safe_dn
 
 
 class NmasGetUniversalPassword(ExtendedOperation):
@@ -37,6 +38,10 @@ class NmasGetUniversalPassword(ExtendedOperation):
 
     def __init__(self, connection, user, controls=None):
         ExtendedOperation.__init__(self, connection, controls)  # calls super __init__()
+
+        if connection.check_names:
+            user = safe_dn(user)
+
         self.request_value['nmasver'] = NMAS_LDAP_EXT_VERSION
         self.request_value['reqdn'] = user
 
