@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2013 - 2018 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -23,34 +23,36 @@
 # along with ldap3 in the COPYING and COPYING.LESSER files.
 # If not, see <http://www.gnu.org/licenses/>.
 
-from .. import SIMPLE, ANONYMOUS, SASL, RESULT_CODES
-from ..core.exceptions import LDAPPasswordIsMandatoryError, LDAPUnknownAuthenticationMethodError, LDAPUserNameNotAllowedError, \
-    LDAPUserNameIsMandatoryError
+from .. import SIMPLE, ANONYMOUS, SASL, STRING_TYPES
+from ..core.results import RESULT_CODES
+from ..core.exceptions import LDAPPasswordIsMandatoryError, LDAPUnknownAuthenticationMethodError, LDAPUserNameNotAllowedError
 from ..protocol.sasl.sasl import validate_simple_password
 from ..protocol.rfc4511 import Version, AuthenticationChoice, Simple, BindRequest, ResultCode, SaslCredentials, BindResponse, \
     LDAPDN, LDAPString, Referral, ServerSaslCreds, SicilyPackageDiscovery, SicilyNegotiate, SicilyResponse
 from ..protocol.convert import authentication_choice_to_dict, referrals_to_list
+from ..utils.conv import to_unicode, to_raw
 
-# BindRequest ::= [APPLICATION 0] SEQUENCE {
-#                                           version        INTEGER (1 ..  127),
-#                                           name           LDAPDN,
-#                                           authentication AuthenticationChoice }
-
-
+# noinspection PyUnresolvedReferences
 def bind_operation(version,
                    authentication,
                    name='',
                    password=None,
                    sasl_mechanism=None,
-                   sasl_credentials=None):
+                   sasl_credentials=None,
+                   auto_encode=False):
+    # BindRequest ::= [APPLICATION 0] SEQUENCE {
+    #                                           version        INTEGER (1 ..  127),
+    #                                           name           LDAPDN,
+    #                                           authentication AuthenticationChoice }
     request = BindRequest()
     request['version'] = Version(version)
     if name is None:
         name = ''
-    request['name'] = name
+    if isinstance(name, STRING_TYPES):
+        request['name'] = to_unicode(name) if auto_encode else name
     if authentication == SIMPLE:
         if not name:
-            raise LDAPUserNameIsMandatoryError('user name is mandatory in simple bind')
+            raise LDAPPasswordIsMandatoryError('user name is mandatory in simple bind')
         if password:
             request['authentication'] = AuthenticationChoice().setComponentByName('simple', Simple(validate_simple_password(password)))
         else:
@@ -60,6 +62,8 @@ def bind_operation(version,
         sasl_creds['mechanism'] = sasl_mechanism
         if sasl_credentials is not None:
             sasl_creds['credentials'] = sasl_credentials
+        # else:
+            # sasl_creds['credentials'] = None
         request['authentication'] = AuthenticationChoice().setComponentByName('sasl', sasl_creds)
     elif authentication == ANONYMOUS:
         if name:
@@ -91,17 +95,15 @@ def bind_request_to_dict(request):
             'name': str(request['name']),
             'authentication': authentication_choice_to_dict(request['authentication'])}
 
-# BindResponse ::= [APPLICATION 1] SEQUENCE {
-#                                            COMPONENTS OF LDAPResult,
-#                                            serverSaslCreds    [7] OCTET STRING OPTIONAL }
-
 
 def bind_response_operation(result_code,
                             matched_dn='',
                             diagnostic_message='',
                             referral=None,
                             server_sasl_credentials=None):
-
+    # BindResponse ::= [APPLICATION 1] SEQUENCE {
+    #                                            COMPONENTS OF LDAPResult,
+    #                                            serverSaslCreds    [7] OCTET STRING OPTIONAL }
     response = BindResponse()
     response['resultCode'] = ResultCode(result_code)
     response['matchedDN'] = LDAPDN(matched_dn)
@@ -121,7 +123,7 @@ def bind_response_to_dict(response):
             'dn': str(response['matchedDN']),
             'message': str(response['diagnosticMessage']),
             'referrals': referrals_to_list(response['referral']),
-            'saslCreds': bytes(response['serverSaslCreds']) if response['serverSaslCreds'] is not None else None}
+            'saslCreds': bytes(response['serverSaslCreds']) if response['serverSaslCreds'] is not None and response['serverSaslCreds'].hasValue() else None}
 
 
 def sicily_bind_response_to_dict(response):
@@ -135,8 +137,8 @@ def bind_response_to_dict_fast(response):
     response_dict = dict()
     response_dict['result'] = int(response[0][3])  # resultCode
     response_dict['description'] = RESULT_CODES[response_dict['result']]
-    response_dict['dn'] = response[1][3].decode('utf-8')  # matchedDN
-    response_dict['message'] = response[2][3].decode('utf-8')  # diagnosticMessage
+    response_dict['dn'] = to_unicode(response[1][3], from_server=True)  # matchedDN
+    response_dict['message'] = to_unicode(response[2][3], from_server=True)  # diagnosticMessage
     response_dict['referrals'] = None  # referrals
     response_dict['saslCreds'] = None  # saslCreds
     for r in response[3:]:
@@ -153,6 +155,6 @@ def sicily_bind_response_to_dict_fast(response):
     response_dict['result'] = int(response[0][3])  # resultCode
     response_dict['description'] = RESULT_CODES[response_dict['result']]
     response_dict['server_creds'] = bytes(response[1][3])  # server_creds
-    response_dict['error_message'] = response[2][3].decode('utf-8')  # error_message
+    response_dict['error_message'] = to_unicode(response[2][3], from_server=True)  # error_message
 
     return response_dict
