@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2013, 2014, 2015, 2016, 2017 Giovanni Cannata
+# Copyright 2013 - 2018 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -28,8 +28,9 @@ from pyasn1.type.base import Asn1Item
 
 from ..core.results import RESULT_CODES
 from ..protocol.rfc4511 import ExtendedRequest, RequestName, ResultCode, RequestValue
-from ..protocol.convert import decode_referrals, referrals_to_list
-from ..utils.asn1 import encoder
+from ..protocol.convert import referrals_to_list
+from ..utils.asn1 import encode
+from ..utils.conv import to_unicode
 
 # ExtendedRequest ::= [APPLICATION 23] SEQUENCE {
 #     requestName      [0] LDAPOID,
@@ -42,30 +43,31 @@ def extended_operation(request_name,
     request = ExtendedRequest()
     request['requestName'] = RequestName(request_name)
     if request_value and isinstance(request_value, Asn1Item):
-        request['requestValue'] = RequestValue(encoder.encode(request_value))
+        request['requestValue'] = RequestValue(encode(request_value))
     elif str is not bytes and isinstance(request_value, (bytes, bytearray)):  # in Python 3 doesn't try to encode a byte value
         request['requestValue'] = request_value
     elif request_value and no_encode:  # doesn't encode the value
         request['requestValue'] = request_value
     elif request_value:  # tries to encode as a octet string
-        request['requestValue'] = RequestValue(encoder.encode(OctetString(str(request_value))))
+        request['requestValue'] = RequestValue(encode(OctetString(str(request_value))))
+
     # elif request_value is not None:
     #     raise LDAPExtensionError('unable to encode value for extended operation')
     return request
 
 
 def extended_request_to_dict(request):
-    return {'name': str(request['requestName']), 'value': bytes(request['requestValue']) if request['requestValue'] else None}
-
+    # return {'name': str(request['requestName']), 'value': bytes(request['requestValue']) if request['requestValue'] else None}
+    return {'name': str(request['requestName']), 'value': bytes(request['requestValue']) if 'requestValue' in request and request['requestValue'] is not None and request['requestValue'].hasValue()  else None}
 
 def extended_response_to_dict(response):
     return {'result': int(response['resultCode']),
             'dn': str(response['matchedDN']),
             'message': str(response['diagnosticMessage']),
             'description': ResultCode().getNamedValues().getName(response['resultCode']),
-            'referrals': decode_referrals(response['referral']),
+            'referrals': referrals_to_list(response['referral']),
             'responseName': str(response['responseName']) if response['responseName'] else None,
-            'responseValue': bytes(response['responseValue']) if response['responseValue'] else bytes()}
+            'responseValue': bytes(response['responseValue']) if response['responseValue'] is not None and response['responseValue'].hasValue() else bytes()}
 
 
 def intermediate_response_to_dict(response):
@@ -77,8 +79,8 @@ def extended_response_to_dict_fast(response):
     response_dict = dict()
     response_dict['result'] = int(response[0][3])  # resultCode
     response_dict['description'] = RESULT_CODES[response_dict['result']]
-    response_dict['dn'] = response[1][3].decode('utf-8')  # matchedDN
-    response_dict['message'] = response[2][3].decode('utf-8')  # diagnosticMessage
+    response_dict['dn'] = to_unicode(response[1][3], from_server=True)  # matchedDN
+    response_dict['message'] = to_unicode(response[2][3], from_server=True)  # diagnosticMessage
     response_dict['referrals'] = None  # referrals
     response_dict['responseName'] = None  # referrals
     response_dict['responseValue'] = None  # responseValue
@@ -87,7 +89,7 @@ def extended_response_to_dict_fast(response):
         if r[2] == 3:  # referrals
             response_dict['referrals'] = referrals_to_list(r[3])  # referrals
         elif r[2] == 10:  # responseName
-            response_dict['responseName'] = r[3].decode('utf-8')
+            response_dict['responseName'] = to_unicode(r[3], from_server=True)
             response_dict['responseValue'] = b''  # responseValue could be empty
 
         else:  # responseValue (11)
@@ -100,7 +102,7 @@ def intermediate_response_to_dict_fast(response):
     response_dict = dict()
     for r in response:
         if r[2] == 0:  # responseName
-            response_dict['responseName'] = r[3].decode('utf-8')
+            response_dict['responseName'] = to_unicode(r[3], from_server=True)
         else:  # responseValue (1)
             response_dict['responseValue'] = bytes(r[3])
 
