@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2013 - 2018 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -26,37 +26,19 @@
 from string import whitespace
 from os import linesep
 
-from .. import DEREF_NEVER, BASE, LEVEL, SUBTREE, DEREF_SEARCH, DEREF_BASE, DEREF_ALWAYS, NO_ATTRIBUTES, ATTRIBUTES_EXCLUDED_FROM_CHECK, \
-    SEQUENCE_TYPES, get_config_parameter
+from .. import DEREF_NEVER, BASE, LEVEL, SUBTREE, DEREF_SEARCH, DEREF_BASE, DEREF_ALWAYS, NO_ATTRIBUTES, SEQUENCE_TYPES, get_config_parameter, STRING_TYPES
 
 from ..core.exceptions import LDAPInvalidFilterError, LDAPAttributeError, LDAPInvalidScopeError, LDAPInvalidDereferenceAliasesError
-from ..protocol.formatters.formatters import format_unicode
 from ..utils.ciDict import CaseInsensitiveDict
 from ..protocol.rfc4511 import SearchRequest, LDAPDN, Scope, DerefAliases, Integer0ToMax, TypesOnly, \
     AttributeSelection, Selector, EqualityMatch, AttributeDescription, AssertionValue, Filter, \
     Not, And, Or, ApproxMatch, GreaterOrEqual, LessOrEqual, ExtensibleMatch, Present, SubstringFilter, \
     Substrings, Final, Initial, Any, ResultCode, Substring, MatchingRule, Type, MatchValue, DnAttributes
 from ..operation.bind import referrals_to_list
-from ..protocol.convert import ava_to_dict, attributes_to_list, search_refs_to_list, validate_assertion_value
+from ..protocol.convert import ava_to_dict, attributes_to_list, search_refs_to_list, validate_assertion_value, prepare_filter_for_sending, search_refs_to_list_fast
 from ..protocol.formatters.standard import format_attribute_values
+from ..utils.conv import to_unicode, to_raw
 
-# SearchRequest ::= [APPLICATION 3] SEQUENCE {
-# baseObject      LDAPDN,
-#     scope           ENUMERATED {
-#         baseObject              (0),
-#         singleLevel             (1),
-#         wholeSubtree            (2),
-#     ...  },
-#     derefAliases    ENUMERATED {
-#         neverDerefAliases       (0),
-#         derefInSearching        (1),
-#         derefFindingBaseObj     (2),
-#         derefAlways             (3) },
-#     sizeLimit       INTEGER (0 ..  maxInt),
-#     timeLimit       INTEGER (0 ..  maxInt),
-#     typesOnly       BOOLEAN,
-#     filter          Filter,
-#     attributes      AttributeSelection }
 
 ROOT = 0
 AND = 1
@@ -101,7 +83,7 @@ class FilterNode(object):
         return representation
 
 
-def evaluate_match(match, schema):
+def evaluate_match(match, schema, auto_escape, auto_encode, check_names):
     left_part, equal_sign, right_part = match.strip().partition('=')
     if not equal_sign:
         raise LDAPInvalidFilterError('invalid matching assertion')
@@ -109,25 +91,25 @@ def evaluate_match(match, schema):
         tag = MATCH_APPROX
         left_part = left_part[:-1].strip()
         right_part = right_part.strip()
-        assertion = {'attr': left_part, 'value': validate_assertion_value(schema, left_part, right_part)}
+        assertion = {'attr': left_part, 'value': validate_assertion_value(schema, left_part, right_part, auto_escape, auto_encode, check_names)}
     elif left_part.endswith('>'):  # greater or equal match '>='
         tag = MATCH_GREATER_OR_EQUAL
         left_part = left_part[:-1].strip()
         right_part = right_part.strip()
-        assertion = {'attr': left_part, 'value': validate_assertion_value(schema, left_part, right_part)}
-    elif left_part.endswith('<'):  # # less or equal match '<='
+        assertion = {'attr': left_part, 'value': validate_assertion_value(schema, left_part, right_part, auto_escape, auto_encode, check_names)}
+    elif left_part.endswith('<'):  # less or equal match '<='
         tag = MATCH_LESS_OR_EQUAL
         left_part = left_part[:-1].strip()
         right_part = right_part.strip()
-        assertion = {'attr': left_part, 'value': validate_assertion_value(schema, left_part, right_part)}
+        assertion = {'attr': left_part, 'value': validate_assertion_value(schema, left_part, right_part, auto_escape, auto_encode, check_names)}
     elif left_part.endswith(':'):  # extensible match ':='
         tag = MATCH_EXTENSIBLE
         left_part = left_part[:-1].strip()
         right_part = right_part.strip()
         extended_filter_list = left_part.split(':')
-        matching_rule = None
-        dn_attributes = None
-        attribute_name = None
+        matching_rule = False
+        dn_attributes = False
+        attribute_name = False
         if extended_filter_list[0] == '':  # extensible filter format [:dn]:matchingRule:=assertionValue
             if len(extended_filter_list) == 2 and extended_filter_list[1].lower().strip() != 'dn':
                 matching_rule = extended_filter_list[1]
@@ -154,32 +136,41 @@ def evaluate_match(match, schema):
 
         if not attribute_name and not matching_rule:
             raise LDAPInvalidFilterError('invalid extensible filter')
-        attribute_name = attribute_name.strip() if attribute_name else None
-        matching_rule = matching_rule.strip() if matching_rule else None
-        assertion = {'attr': attribute_name, 'value': validate_assertion_value(schema, attribute_name, right_part), 'matchingRule': matching_rule, 'dnAttributes': dn_attributes}
-    elif right_part == '*':  # # attribute present match '=*'
+        attribute_name = attribute_name.strip() if attribute_name else False
+        matching_rule = matching_rule.strip() if matching_rule else False
+        assertion = {'attr': attribute_name, 'value': validate_assertion_value(schema, attribute_name, right_part, auto_escape, auto_encode, check_names), 'matchingRule': matching_rule, 'dnAttributes': dn_attributes}
+    elif right_part == '*':  # attribute present match '=*'
         tag = MATCH_PRESENT
         left_part = left_part.strip()
         assertion = {'attr': left_part}
-    elif '*' in right_part:  # # substring match '=initial*substring*substring*final'
+    elif '*' in right_part:  # substring match '=initial*substring*substring*final'
         tag = MATCH_SUBSTRING
         left_part = left_part.strip()
         right_part = right_part.strip()
         substrings = right_part.split('*')
-        initial = validate_assertion_value(schema, left_part, substrings[0]) if substrings[0] else None
-        final = validate_assertion_value(schema, left_part, substrings[-1]) if substrings[-1] else None
-        any_string = [validate_assertion_value(schema, left_part, substring) for substring in substrings[1:-1] if substring]
-        assertion = {'attr': left_part, 'initial': initial, 'any': any_string, 'final': final}
+        initial = validate_assertion_value(schema, left_part, substrings[0], auto_escape, auto_encode, check_names) if substrings[0] else None
+        final = validate_assertion_value(schema, left_part, substrings[-1], auto_escape, auto_encode, check_names) if substrings[-1] else None
+        any_string = [validate_assertion_value(schema, left_part, substring, auto_escape, auto_encode, check_names) for substring in substrings[1:-1] if substring]
+        #assertion = {'attr': left_part, 'initial': initial, 'any': any_string, 'final': final}
+        assertion = {'attr': left_part}
+        if initial:
+            assertion['initial'] =  initial
+        if any_string:
+            assertion['any'] = any_string
+        if final:
+            assertion['final'] =  final
     else:  # equality match '='
         tag = MATCH_EQUAL
         left_part = left_part.strip()
         right_part = right_part.strip()
-        assertion = {'attr': left_part, 'value': validate_assertion_value(schema, left_part, right_part)}
+        assertion = {'attr': left_part, 'value': validate_assertion_value(schema, left_part, right_part, auto_escape, auto_encode, check_names)}
 
     return FilterNode(tag, assertion)
 
 
-def parse_filter(search_filter, schema):
+def parse_filter(search_filter, schema, auto_escape, auto_encode, check_names):
+    if str != bytes and isinstance(search_filter, bytes):  # python 3 with byte filter
+        search_filter = to_unicode(search_filter)
     search_filter = search_filter.strip()
     if search_filter and search_filter.count('(') == search_filter.count(')') and search_filter.startswith('(') and search_filter.endswith(')'):
         state = SEARCH_OPEN_OR_CLOSE
@@ -211,8 +202,8 @@ def parse_filter(search_filter, schema):
                     end_pos = pos
                     if start_pos:
                         if current_node.tag == NOT and len(current_node.elements) > 0:
-                            raise LDAPInvalidFilterError('not clause in filter cannot be multiple')
-                        current_node.append(evaluate_match(search_filter[start_pos:end_pos], schema))
+                            raise LDAPInvalidFilterError('NOT (!) clause in filter cannot be multiple')
+                        current_node.append(evaluate_match(search_filter[start_pos:end_pos], schema, auto_escape, auto_encode, check_names))
                 start_pos = None
                 state = SEARCH_OPEN_OR_CLOSE
             elif (state == SEARCH_MATCH_OR_CLOSE or state == SEARCH_MATCH_OR_CONTROL) and c not in '()':
@@ -230,6 +221,7 @@ def parse_filter(search_filter, schema):
 
 
 def compile_filter(filter_node):
+    """Builds ASN1 structure for filter, converts from filter LDAP escaping to bytes"""
     compiled_filter = Filter()
     if filter_node.tag == AND:
         boolean_filter = And()
@@ -248,23 +240,22 @@ def compile_filter(filter_node):
     elif filter_node.tag == NOT:
         boolean_filter = Not()
         boolean_filter['innerNotFilter'] = compile_filter(filter_node.elements[0])
-        # compiled_filter['notFilter'] = boolean_filter
         compiled_filter.setComponentByName('notFilter', boolean_filter, verifyConstraints=False)  # do not verify constraints because of hack for recursive filters in rfc4511
 
     elif filter_node.tag == MATCH_APPROX:
         matching_filter = ApproxMatch()
         matching_filter['attributeDesc'] = AttributeDescription(filter_node.assertion['attr'])
-        matching_filter['assertionValue'] = AssertionValue(filter_node.assertion['value'])
+        matching_filter['assertionValue'] = AssertionValue(prepare_filter_for_sending(filter_node.assertion['value']))
         compiled_filter['approxMatch'] = matching_filter
     elif filter_node.tag == MATCH_GREATER_OR_EQUAL:
         matching_filter = GreaterOrEqual()
         matching_filter['attributeDesc'] = AttributeDescription(filter_node.assertion['attr'])
-        matching_filter['assertionValue'] = AssertionValue(filter_node.assertion['value'])
+        matching_filter['assertionValue'] = AssertionValue(prepare_filter_for_sending(filter_node.assertion['value']))
         compiled_filter['greaterOrEqual'] = matching_filter
     elif filter_node.tag == MATCH_LESS_OR_EQUAL:
         matching_filter = LessOrEqual()
         matching_filter['attributeDesc'] = AttributeDescription(filter_node.assertion['attr'])
-        matching_filter['assertionValue'] = AssertionValue(filter_node.assertion['value'])
+        matching_filter['assertionValue'] = AssertionValue(prepare_filter_for_sending(filter_node.assertion['value']))
         compiled_filter['lessOrEqual'] = matching_filter
     elif filter_node.tag == MATCH_EXTENSIBLE:
         matching_filter = ExtensibleMatch()
@@ -272,7 +263,7 @@ def compile_filter(filter_node):
             matching_filter['matchingRule'] = MatchingRule(filter_node.assertion['matchingRule'])
         if filter_node.assertion['attr']:
             matching_filter['type'] = Type(filter_node.assertion['attr'])
-        matching_filter['matchValue'] = MatchValue(filter_node.assertion['value'])
+        matching_filter['matchValue'] = MatchValue(prepare_filter_for_sending(filter_node.assertion['value']))
         matching_filter['dnAttributes'] = DnAttributes(filter_node.assertion['dnAttributes'])
         compiled_filter['extensibleMatch'] = matching_filter
     elif filter_node.tag == MATCH_PRESENT:
@@ -283,21 +274,21 @@ def compile_filter(filter_node):
         matching_filter['type'] = AttributeDescription(filter_node.assertion['attr'])
         substrings = Substrings()
         pos = 0
-        if filter_node.assertion['initial']:
-            substrings[pos] = Substring().setComponentByName('initial', Initial(filter_node.assertion['initial']))
+        if 'initial' in filter_node.assertion and filter_node.assertion['initial']:
+            substrings[pos] = Substring().setComponentByName('initial', Initial(prepare_filter_for_sending(filter_node.assertion['initial'])))
             pos += 1
-        if filter_node.assertion['any']:
+        if 'any' in filter_node.assertion and filter_node.assertion['any']:
             for substring in filter_node.assertion['any']:
-                substrings[pos] = Substring().setComponentByName('any', Any(substring))
+                substrings[pos] = Substring().setComponentByName('any', Any(prepare_filter_for_sending(substring)))
                 pos += 1
-        if filter_node.assertion['final']:
-            substrings[pos] = Substring().setComponentByName('final', Final(filter_node.assertion['final']))
+        if 'final' in filter_node.assertion and filter_node.assertion['final']:
+            substrings[pos] = Substring().setComponentByName('final', Final(prepare_filter_for_sending(filter_node.assertion['final'])))
         matching_filter['substrings'] = substrings
         compiled_filter['substringFilter'] = matching_filter
     elif filter_node.tag == MATCH_EQUAL:
         matching_filter = EqualityMatch()
         matching_filter['attributeDesc'] = AttributeDescription(filter_node.assertion['attr'])
-        matching_filter['assertionValue'] = AssertionValue(filter_node.assertion['value'])
+        matching_filter['assertionValue'] = AssertionValue(prepare_filter_for_sending(filter_node.assertion['value']))
         compiled_filter.setComponentByName('equalityMatch', matching_filter)
     else:
         raise LDAPInvalidFilterError('unknown filter node tag')
@@ -305,19 +296,17 @@ def compile_filter(filter_node):
     return compiled_filter
 
 
-def build_filter(search_filter, schema):
-    return compile_filter(parse_filter(search_filter, schema).elements[0])
-
-
 def build_attribute_selection(attribute_list, schema):
+    conf_attributes_excluded_from_check = [v.lower() for v in get_config_parameter('ATTRIBUTES_EXCLUDED_FROM_CHECK')]
+
     attribute_selection = AttributeSelection()
     for index, attribute in enumerate(attribute_list):
-        if schema and schema.attribute_types is not None:
+        if schema and schema.attribute_types:
             if ';' in attribute:  # exclude tags from validation
-                if not attribute[0:attribute.index(';')] in schema.attribute_types and attribute not in ATTRIBUTES_EXCLUDED_FROM_CHECK:
+                if not attribute[0:attribute.index(';')] in schema.attribute_types and attribute.lower() not in conf_attributes_excluded_from_check:
                     raise LDAPAttributeError('invalid attribute type in attribute list: ' + attribute)
             else:
-                if attribute not in schema.attribute_types and attribute not in ATTRIBUTES_EXCLUDED_FROM_CHECK:
+                if attribute not in schema.attribute_types and attribute.lower() not in conf_attributes_excluded_from_check:
                     raise LDAPAttributeError('invalid attribute type in attribute list: ' + attribute)
         attribute_selection[index] = Selector(attribute)
 
@@ -332,8 +321,27 @@ def search_operation(search_base,
                      size_limit,
                      time_limit,
                      types_only,
-                     schema=None):
-
+                     auto_escape,
+                     auto_encode,
+                     schema=None,
+                     check_names=False):
+    # SearchRequest ::= [APPLICATION 3] SEQUENCE {
+    # baseObject      LDAPDN,
+    #     scope           ENUMERATED {
+    #         baseObject              (0),
+    #         singleLevel             (1),
+    #         wholeSubtree            (2),
+    #     ...  },
+    #     derefAliases    ENUMERATED {
+    #         neverDerefAliases       (0),
+    #         derefInSearching        (1),
+    #         derefFindingBaseObj     (2),
+    #         derefAlways             (3) },
+    #     sizeLimit       INTEGER (0 ..  maxInt),
+    #     timeLimit       INTEGER (0 ..  maxInt),
+    #     typesOnly       BOOLEAN,
+    #     filter          Filter,
+    #     attributes      AttributeSelection }
     request = SearchRequest()
     request['baseObject'] = LDAPDN(search_base)
 
@@ -360,7 +368,7 @@ def search_operation(search_base,
     request['sizeLimit'] = Integer0ToMax(size_limit)
     request['timeLimit'] = Integer0ToMax(time_limit)
     request['typesOnly'] = TypesOnly(True) if types_only else TypesOnly(False)
-    request['filter'] = build_filter(search_filter, schema)  # parse the searchFilter string and compile it starting from the root node
+    request['filter'] = compile_filter(parse_filter(search_filter, schema, auto_escape, auto_encode, check_names).elements[0])  # parse the searchFilter string and compile it starting from the root node
     if not isinstance(attributes, SEQUENCE_TYPES):
         attributes = [NO_ATTRIBUTES]
 
@@ -374,11 +382,15 @@ def decode_vals(vals):
 
 
 def decode_vals_fast(vals):
-    return [val[3].decode('utf-8') for val in vals if val] if vals else None
+    try:
+        return [to_unicode(val[3], from_server=True) for val in vals if val] if vals else None
+    except UnicodeDecodeError:
+        return [val[3] for val in vals if val] if vals else None
 
 
 def attributes_to_dict(attribute_list):
-    attributes = CaseInsensitiveDict() if get_config_parameter('CASE_INSENSITIVE_ATTRIBUTE_NAMES') else dict()
+    conf_case_insensitive_attributes = get_config_parameter('CASE_INSENSITIVE_ATTRIBUTE_NAMES')
+    attributes = CaseInsensitiveDict() if conf_case_insensitive_attributes else dict()
     for attribute in attribute_list:
         attributes[str(attribute['type'])] = decode_vals(attribute['vals'])
 
@@ -386,9 +398,10 @@ def attributes_to_dict(attribute_list):
 
 
 def attributes_to_dict_fast(attribute_list):
-    attributes = CaseInsensitiveDict() if get_config_parameter('CASE_INSENSITIVE_ATTRIBUTE_NAMES') else dict()
+    conf_case_insensitive_attributes = get_config_parameter('CASE_INSENSITIVE_ATTRIBUTE_NAMES')
+    attributes = CaseInsensitiveDict() if conf_case_insensitive_attributes else dict()
     for attribute in attribute_list:
-        attributes[attribute[3][0][3].decode('utf-8')] = decode_vals_fast(attribute[3][1][3])
+        attributes[to_unicode(attribute[3][0][3], from_server=True)] = decode_vals_fast(attribute[3][1][3])
 
     return attributes
 
@@ -402,7 +415,9 @@ def decode_raw_vals_fast(vals):
 
 
 def raw_attributes_to_dict(attribute_list):
-    attributes = CaseInsensitiveDict() if get_config_parameter('CASE_INSENSITIVE_ATTRIBUTE_NAMES') else dict()
+    conf_case_insensitive_attributes = get_config_parameter('CASE_INSENSITIVE_ATTRIBUTE_NAMES')
+
+    attributes = CaseInsensitiveDict() if conf_case_insensitive_attributes else dict()
     for attribute in attribute_list:
         attributes[str(attribute['type'])] = decode_raw_vals(attribute['vals'])
 
@@ -410,15 +425,18 @@ def raw_attributes_to_dict(attribute_list):
 
 
 def raw_attributes_to_dict_fast(attribute_list):
-    attributes = CaseInsensitiveDict() if get_config_parameter('CASE_INSENSITIVE_ATTRIBUTE_NAMES') else dict()
+    conf_case_insensitive_attributes = get_config_parameter('CASE_INSENSITIVE_ATTRIBUTE_NAMES')
+    attributes = CaseInsensitiveDict() if conf_case_insensitive_attributes else dict()
     for attribute in attribute_list:
-        attributes[attribute[3][0][3].decode('utf-8')] = decode_raw_vals_fast(attribute[3][1][3])
+        attributes[to_unicode(attribute[3][0][3], from_server=True)] = decode_raw_vals_fast(attribute[3][1][3])
 
     return attributes
 
 
 def checked_attributes_to_dict(attribute_list, schema=None, custom_formatter=None):
-    checked_attributes = CaseInsensitiveDict() if get_config_parameter('CASE_INSENSITIVE_ATTRIBUTE_NAMES') else dict()
+    conf_case_insensitive_attributes = get_config_parameter('CASE_INSENSITIVE_ATTRIBUTE_NAMES')
+
+    checked_attributes = CaseInsensitiveDict() if conf_case_insensitive_attributes else dict()
     for attribute in attribute_list:
         name = str(attribute['type'])
         checked_attributes[name] = format_attribute_values(schema, name, decode_raw_vals(attribute['vals']) or [], custom_formatter)
@@ -426,9 +444,11 @@ def checked_attributes_to_dict(attribute_list, schema=None, custom_formatter=Non
 
 
 def checked_attributes_to_dict_fast(attribute_list, schema=None, custom_formatter=None):
-    checked_attributes = CaseInsensitiveDict() if get_config_parameter('CASE_INSENSITIVE_ATTRIBUTE_NAMES') else dict()
+    conf_case_insensitive_attributes = get_config_parameter('CASE_INSENSITIVE_ATTRIBUTE_NAMES')
+
+    checked_attributes = CaseInsensitiveDict() if conf_case_insensitive_attributes else dict()
     for attribute in attribute_list:
-        name = attribute[3][0][3].decode('utf-8')
+        name = to_unicode(attribute[3][0][3], from_server=True)
         checked_attributes[name] = format_attribute_values(schema, name, decode_raw_vals_fast(attribute[3][1][3]) or [], custom_formatter)
     return checked_attributes
 
@@ -445,7 +465,7 @@ def filter_to_string(filter_object):
         for f in filter_object['and']:
             filter_string += filter_to_string(f)
     elif filter_type == 'or':
-        filter_string += '!'
+        filter_string += '|'
         for f in filter_object['or']:
             filter_string += filter_to_string(f)
     elif filter_type == 'notFilter':
@@ -457,13 +477,15 @@ def filter_to_string(filter_object):
         attribute = filter_object['substringFilter']['type']
         filter_string += str(attribute) + '='
         for substring in filter_object['substringFilter']['substrings']:
-            if substring['initial']:
-                filter_string += str(substring['initial']) + '*'
-            elif substring['any']:
-                filter_string += str(substring['any']) if filter_string.endswith('*') else '*' + str(substring['any'])
-                filter_string += '*'
-            elif substring['final']:
-                filter_string += '*' + str(substring['final'])
+            component = substring.getName()
+            if substring[component] is not None and substring[component].hasValue():
+                if component == 'initial':
+                    filter_string += str(substring['initial']) + '*'
+                elif component == 'any':
+                    filter_string += str(substring['any']) if filter_string.endswith('*') else '*' + str(substring['any'])
+                    filter_string += '*'
+                elif component == 'final':
+                    filter_string += '*' + str(substring['final'])
     elif filter_type == 'greaterOrEqual':
         ava = ava_to_dict(filter_object['greaterOrEqual'])
         filter_string += ava['attribute'] + '>=' + ava['value']
@@ -490,7 +512,7 @@ def search_request_to_dict(request):
             'dereferenceAlias': int(request['derefAliases']),
             'sizeLimit': int(request['sizeLimit']),
             'timeLimit': int(request['timeLimit']),
-            'typeOnly': bool(request['typesOnly']),
+            'typesOnly': bool(request['typesOnly']),
             'filter': filter_to_string(request['filter']),
             'attributes': attributes_to_list(request['attributes'])}
 
@@ -498,7 +520,15 @@ def search_request_to_dict(request):
 def search_result_entry_response_to_dict(response, schema, custom_formatter, check_names):
     entry = dict()
     # entry['dn'] = str(response['object'])
-    entry['dn'] = format_unicode(str(response['object']))
+    if response['object']:
+        entry['raw_dn'] = to_raw(response['object'])
+        if isinstance(response['object'], STRING_TYPES):  # mock strategies return string not a PyAsn1 object
+            entry['dn'] = to_unicode(response['object'])
+        else:
+            entry['dn'] = to_unicode(bytes(response['object']), from_server=True)
+    else:
+        entry['raw_dn'] = b''
+        entry['dn'] = ''
     entry['raw_attributes'] = raw_attributes_to_dict(response['attributes'])
     if check_names:
         entry['attributes'] = checked_attributes_to_dict(response['attributes'], schema, custom_formatter)
@@ -509,20 +539,26 @@ def search_result_entry_response_to_dict(response, schema, custom_formatter, che
 
 
 def search_result_done_response_to_dict(response):
-    return {'result': int(response[0]),
-            'description': ResultCode().getNamedValues().getName(response[0]),
+    result = {'result': int(response['resultCode']),
+            'description': ResultCode().getNamedValues().getName(response['resultCode']),
             'message': str(response['diagnosticMessage']),
             'dn': str(response['matchedDN']),
             'referrals': referrals_to_list(response['referral'])}
 
+    if 'controls' in response:  # used for returning controls in Mock strategies
+        result['controls'] = dict()
+        for control in response['controls']:
+            result['controls'][control[0]] = control[1]
 
+    return result
 def search_result_reference_response_to_dict(response):
     return {'uri': search_refs_to_list(response)}
 
 
 def search_result_entry_response_to_dict_fast(response, schema, custom_formatter, check_names):
     entry_dict = dict()
-    entry_dict['dn'] = response[0][3].decode('utf-8')  # object
+    entry_dict['raw_dn'] = response[0][3]
+    entry_dict['dn'] = to_unicode(response[0][3], from_server=True)
     entry_dict['raw_attributes'] = raw_attributes_to_dict_fast(response[1][3])  # attributes
     if check_names:
         entry_dict['attributes'] = checked_attributes_to_dict_fast(response[1][3], schema, custom_formatter)  # attributes
@@ -533,4 +569,4 @@ def search_result_entry_response_to_dict_fast(response, schema, custom_formatter
 
 
 def search_result_reference_response_to_dict_fast(response):
-    return {'uri': search_refs_to_list([r[3] for r in response])}
+    return {'uri': search_refs_to_list_fast([r[3] for r in response])}
