@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2016, 2017 Giovanni Cannata
+# Copyright 2016 - 2018 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -27,6 +27,7 @@ import json
 import re
 
 from threading import Lock
+from random import SystemRandom
 
 from pyasn1.type.univ import OctetString
 
@@ -42,14 +43,21 @@ from ..operation.search import search_request_to_dict, parse_filter, ROOT, AND,
     MATCH_GREATER_OR_EQUAL, MATCH_LESS_OR_EQUAL, MATCH_EXTENSIBLE, MATCH_PRESENT,\
     MATCH_SUBSTRING, MATCH_EQUAL
 from ..utils.conv import json_hook, to_unicode, to_raw
-from ..core.exceptions import LDAPDefinitionError, LDAPPasswordIsMandatoryError, LDAPInvalidValueError
+from ..core.exceptions import LDAPDefinitionError, LDAPPasswordIsMandatoryError, LDAPInvalidValueError, LDAPSocketOpenError
+from ..core.results import RESULT_SUCCESS, RESULT_OPERATIONS_ERROR, RESULT_UNAVAILABLE_CRITICAL_EXTENSION, \
+    RESULT_INVALID_CREDENTIALS, RESULT_NO_SUCH_OBJECT, RESULT_ENTRY_ALREADY_EXISTS, RESULT_COMPARE_TRUE, \
+    RESULT_COMPARE_FALSE, RESULT_NO_SUCH_ATTRIBUTE, RESULT_UNWILLING_TO_PERFORM
 from ..utils.ciDict import CaseInsensitiveDict
 from ..utils.dn import to_dn, safe_dn, safe_rdn
 from ..protocol.sasl.sasl import validate_simple_password
-from ..protocol.formatters.standard import find_attribute_validator
+from ..protocol.formatters.standard import find_attribute_validator, format_attribute_values
+from ..protocol.rfc2696 import paged_search_control
 from ..utils.log import log, log_enabled, ERROR, BASIC
-from ..protocol.formatters.standard import format_attribute_values
-from ..utils.asn1 import encoder
+from ..utils.asn1 import encode
+from ..strategy.base import BaseStrategy  # needed for decode_control() method
+from ..protocol.rfc4511 import LDAPMessage, ProtocolOp, MessageID
+from ..protocol.convert import build_controls_list
+
 
 # LDAPResult ::= SEQUENCE {
 #     resultCode         ENUMERATED {
@@ -105,6 +113,47 @@ from ..utils.asn1 import encoder
 #     referral           [3] Referral OPTIONAL }
 
 # noinspection PyProtectedMember,PyUnresolvedReferences
+
+SEARCH_CONTROLS = ['1.2.840.113556.1.4.319'  # simple paged search [RFC 2696]
+                   ]
+SERVER_ENCODING = 'utf-8'
+
+
+def random_cookie():
+    return to_raw(SystemRandom().random())[-6:]
+
+
+class PagedSearchSet(object):
+    def __init__(self, response, size, criticality):
+        self.size = size
+        self.response = response
+        self.cookie = None
+        self.sent = 0
+        self.done = False
+
+    def next(self, size=None):
+        if size:
+            self.size=size
+
+        message = ''
+        response = self.response[self.sent: self.sent + self.size]
+        self.sent += self.size
+        if self.sent > len(self.response):
+            self.done = True
+            self.cookie = ''
+        else:
+            self.cookie = random_cookie()
+
+        response_control = paged_search_control(False, len(self.response), self.cookie)
+        result = {'resultCode': RESULT_SUCCESS,
+                  'matchedDN': '',
+                  'diagnosticMessage': to_unicode(message, SERVER_ENCODING),
+                  'referral': None,
+                  'controls': [BaseStrategy.decode_control(response_control)]
+                  }
+        return response, result
+
+
 class MockBaseStrategy(object):
     """
     Base class for connection strategy
@@ -112,7 +161,6 @@ class MockBaseStrategy(object):
 
     def __init__(self):
         if not hasattr(self.connection.server, 'dit'):  # create entries dict if not already present
-            self.connection.server.dit_lock = Lock()
             self.connection.server.dit = CaseInsensitiveDict()
         self.entries = self.connection.server.dit  # for simpler reference
         self.no_real_dsa = True
@@ -120,6 +168,7 @@ class MockBaseStrategy(object):
         self.custom_validators = None
         self.operational_attributes = ['entryDN']
         self.add_entry('cn=schema', [])  # add default entry for schema
+        self._paged_sets = []  # list of paged search in progress
         if log_enabled(BASIC):
             log(BASIC, 'instantiated <%s>: <%s>', self.__class__.__name__, self)
 
@@ -161,6 +210,8 @@ class MockBaseStrategy(object):
             if escaped_dn not in self.connection.server.dit:
                 new_entry = CaseInsensitiveDict()
                 for attribute in attributes:
+                    if attribute in self.operational_attributes:  # no restore of operational attributes, should be computed at runtime
+                        continue
                     if not isinstance(attributes[attribute], SEQUENCE_TYPES):  # entry attributes are always lists of bytes values
                         attributes[attribute] = [attributes[attribute]]
                     if self.connection.server.schema and self.connection.server.schema.attribute_types[attribute].single_value and len(attributes[attribute]) > 1:  # multiple values in single-valued attribute
@@ -187,8 +238,9 @@ class MockBaseStrategy(object):
                     if rdn[0] not in new_entry:  # if rdn attribute is missing adds attribute and its value
                         new_entry[rdn[0]] = [to_raw(rdn[1])]
                     else:
-                        if rdn[1] not in new_entry[rdn[0]]:  # add rdn value if rdn attribute is present but value is missing
-                            new_entry[rdn[0]].append(to_raw(rdn[1]))
+                        raw_rdn = to_raw(rdn[1])
+                        if raw_rdn not in new_entry[rdn[0]]:  # add rdn value if rdn attribute is present but value is missing
+                            new_entry[rdn[0]].append(raw_rdn)
                 new_entry['entryDN'] = [to_raw(escaped_dn)]
                 self.connection.server.dit[escaped_dn] = new_entry
                 return True
@@ -256,26 +308,26 @@ class MockBaseStrategy(object):
             if 'userPassword' in self.connection.server.dit[identity]:
                 # if self.connection.server.dit[identity]['userPassword'] == password or password in self.connection.server.dit[identity]['userPassword']:
                 if self.equal(identity, 'userPassword', password):
-                    result_code = 0
+                    result_code = RESULT_SUCCESS
                     message = ''
                     self.bound = identity
                 else:
-                    result_code = 49
+                    result_code = RESULT_INVALID_CREDENTIALS
                     message = 'invalid credentials'
             else:  # no user found, returns invalidCredentials
-                result_code = 49
+                result_code = RESULT_INVALID_CREDENTIALS
                 message = 'missing userPassword attribute'
         elif identity == '<anonymous>':
-            result_code = 0
+            result_code = RESULT_SUCCESS
             message = ''
             self.bound = identity
         else:
-            result_code = 49
+            result_code = RESULT_INVALID_CREDENTIALS
             message = 'missing object'
 
         return {'resultCode': result_code,
                 'matchedDN': '',
-                'diagnosticMessage': to_unicode(message, 'utf-8'),
+                'diagnosticMessage': to_unicode(message, SERVER_ENCODING),
                 'referral': None,
                 'serverSaslCreds': None
                 }
@@ -291,15 +343,15 @@ class MockBaseStrategy(object):
         dn = safe_dn(request['entry'])
         if dn in self.connection.server.dit:
             del self.connection.server.dit[dn]
-            result_code = 0
+            result_code = RESULT_SUCCESS
             message = ''
         else:
-            result_code = 32
+            result_code = RESULT_NO_SUCH_OBJECT
             message = 'object not found'
 
         return {'resultCode': result_code,
                 'matchedDN': '',
-                'diagnosticMessage': to_unicode(message, 'utf-8'),
+                'diagnosticMessage': to_unicode(message, SERVER_ENCODING),
                 'referral': None
                 }
 
@@ -319,18 +371,18 @@ class MockBaseStrategy(object):
 
         if dn not in self.connection.server.dit:
             if self.add_entry(dn, attributes):
-                result_code = 0
+                result_code = RESULT_SUCCESS
                 message = ''
             else:
-                result_code = 1
+                result_code = RESULT_OPERATIONS_ERROR
                 message = 'error adding entry'
         else:
-            result_code = 68
+            result_code = RESULT_ENTRY_ALREADY_EXISTS
             message = 'entry already exist'
 
         return {'resultCode': result_code,
                 'matchedDN': '',
-                'diagnosticMessage': to_unicode(message, 'utf-8'),
+                'diagnosticMessage': to_unicode(message, SERVER_ENCODING),
                 'referral': None
                 }
 
@@ -350,21 +402,21 @@ class MockBaseStrategy(object):
         if dn in self.connection.server.dit:
             if attribute in self.connection.server.dit[dn]:
                 if self.equal(dn, attribute, value):
-                    result_code = 6
+                    result_code = RESULT_COMPARE_TRUE
                     message = ''
                 else:
-                    result_code = 5
+                    result_code = RESULT_COMPARE_FALSE
                     message = ''
             else:
-                result_code = 16
+                result_code = RESULT_NO_SUCH_ATTRIBUTE
                 message = 'attribute not found'
         else:
-            result_code = 32
+            result_code = RESULT_NO_SUCH_OBJECT
             message = 'object not found'
 
         return {'resultCode': result_code,
                 'matchedDN': '',
-                'diagnosticMessage': to_unicode(message, 'utf-8'),
+                'diagnosticMessage': to_unicode(message, SERVER_ENCODING),
                 'referral': None
                 }
 
@@ -389,28 +441,34 @@ class MockBaseStrategy(object):
             if new_superior and new_rdn:  # performs move in the DIT
                 new_dn = safe_dn(dn_components[0] + ',' + new_superior)
                 self.connection.server.dit[new_dn] = self.connection.server.dit[dn].copy()
+                moved_entry = self.connection.server.dit[new_dn]
                 if delete_old_rdn:
                     del self.connection.server.dit[dn]
-                result_code = 0
+                result_code = RESULT_SUCCESS
                 message = 'entry moved'
-                self.connection.server.dit[new_dn]['entryDN'] = [to_raw(new_dn)]
+                moved_entry['entryDN'] = [to_raw(new_dn)]
             elif new_rdn and not new_superior:  # performs rename
                 new_dn = safe_dn(new_rdn + ',' + safe_dn(dn_components[1:]))
                 self.connection.server.dit[new_dn] = self.connection.server.dit[dn].copy()
+                renamed_entry = self.connection.server.dit[new_dn]
                 del self.connection.server.dit[dn]
-                self.connection.server.dit[new_dn]['entryDN'] = [to_raw(new_dn)]
-                result_code = 0
+                renamed_entry['entryDN'] = [to_raw(new_dn)]
+
+                for rdn in safe_rdn(new_dn, decompose=True):  # adds rdns to entry attributes
+                    renamed_entry[rdn[0]] = [to_raw(rdn[1])]
+
+                result_code = RESULT_SUCCESS
                 message = 'entry rdn renamed'
             else:
-                result_code = 53
+                result_code = RESULT_UNWILLING_TO_PERFORM
                 message = 'newRdn or newSuperior missing'
         else:
-            result_code = 32
+            result_code = RESULT_NO_SUCH_OBJECT
             message = 'object not found'
 
         return {'resultCode': result_code,
                 'matchedDN': '',
-                'diagnosticMessage': to_unicode(message, 'utf-8'),
+                'diagnosticMessage': to_unicode(message, SERVER_ENCODING),
                 'referral': None
                 }
 
@@ -460,7 +518,7 @@ class MockBaseStrategy(object):
                             entry[attribute].extend([to_raw(element) for element in elements])
                 elif operation == 1:  # delete
                     if attribute not in entry:  # attribute must exist
-                        result_code = 16
+                        result_code = RESULT_NO_SUCH_ATTRIBUTE
                         message = 'attribute must exists for deleting its values'
                     elif attribute in rdns:  # attribute can't be used in dn
                         result_code = 67
@@ -497,12 +555,12 @@ class MockBaseStrategy(object):
             if result_code:  # an error has happened, restores the original dn
                 self.connection.server.dit[dn] = original_entry
         else:
-            result_code = 32
+            result_code = RESULT_NO_SUCH_OBJECT
             message = 'object not found'
 
         return {'resultCode': result_code,
                 'matchedDN': '',
-                'diagnosticMessage': to_unicode(message, 'utf-8'),
+                'diagnosticMessage': to_unicode(message, SERVER_ENCODING),
                 'referral': None
                 }
 
@@ -539,6 +597,50 @@ class MockBaseStrategy(object):
         # response_entry: object, attributes
         # response_done: LDAPResult
         request = search_request_to_dict(request_message)
+        if controls:
+            decoded_controls = [self.decode_control(control) for control in controls if control]
+            for decoded_control in decoded_controls:
+                if decoded_control[1]['criticality'] and decoded_control[0] not in SEARCH_CONTROLS:
+                    message = 'Critical requested control ' + str(decoded_control[0]) + ' not available'
+                    result = {'resultCode': RESULT_UNAVAILABLE_CRITICAL_EXTENSION,
+                              'matchedDN': '',
+                              'diagnosticMessage': to_unicode(message, SERVER_ENCODING),
+                              'referral': None
+                              }
+                    return [], result
+                elif decoded_control[0] == '1.2.840.113556.1.4.319':  # Simple paged search
+                    if not decoded_control[1]['value']['cookie']:  # new paged search
+                        response, result =  self._execute_search(request)
+                        if result['resultCode'] == RESULT_SUCCESS:  # success
+                            paged_set = PagedSearchSet(response, int(decoded_control[1]['value']['size']), decoded_control[1]['criticality'])
+                            response, result = paged_set.next()
+                            if paged_set.done:  # paged search already completed, no need to store the set
+                                del paged_set
+                            else:
+                                self._paged_sets.append(paged_set)
+                            return response, result
+                        else:
+                            return [], result
+                    else:
+                        for paged_set in self._paged_sets:
+                            if paged_set.cookie == decoded_control[1]['value']['cookie']: # existing paged set
+                                response, result = paged_set.next()  # returns next bunch of entries as per paged set specifications
+                                if paged_set.done:
+                                    self._paged_sets.remove(paged_set)
+                                return response, result
+                        # paged set not found
+                        message = 'Invalid cookie in simple paged search'
+                        result = {'resultCode': RESULT_OPERATIONS_ERROR,
+                                  'matchedDN': '',
+                                  'diagnosticMessage': to_unicode(message, SERVER_ENCODING),
+                                  'referral': None
+                                  }
+                        return [], result
+
+        else:
+            return self._execute_search(request)
+
+    def _execute_search(self, request):
         responses = []
         base = safe_dn(request['base'])
         scope = request['scope']
@@ -548,7 +650,7 @@ class MockBaseStrategy(object):
             attributes.remove('+')
         attributes = [attr.lower() for attr in request['attributes']]
 
-        filter_root = parse_filter(request['filter'], self.connection.server.schema, auto_escape=True, auto_encode=False)
+        filter_root = parse_filter(request['filter'], self.connection.server.schema, auto_escape=True, auto_encode=False, check_names=self.connection.check_names)
         candidates = []
         if scope == 0:  # base object
             if base in self.connection.server.dit or base.lower() == 'cn=schema':
@@ -563,7 +665,7 @@ class MockBaseStrategy(object):
                     candidates.append(entry)
 
         if not candidates:  # incorrect base
-            result_code = 32
+            result_code = RESULT_NO_SUCH_OBJECT
             message = 'incorrect base object'
         else:
             matched = self.evaluate_filter_node(filter_root, candidates)
@@ -581,7 +683,7 @@ class MockBaseStrategy(object):
 
         result = {'resultCode': result_code,
                   'matchedDN': '',
-                  'diagnosticMessage': to_unicode(message, 'utf-8'),
+                  'diagnosticMessage': to_unicode(message, SERVER_ENCODING),
                   'referral': None
                   }
 
@@ -602,7 +704,7 @@ class MockBaseStrategy(object):
         #     responseValue    [1] OCTET STRING OPTIONAL }
         request = extended_request_to_dict(request_message)
 
-        result_code = 53
+        result_code = RESULT_UNWILLING_TO_PERFORM
         message = 'not implemented'
         response_name = None
         response_value = None
@@ -613,17 +715,17 @@ class MockBaseStrategy(object):
                         result_code = 0
                         message = ''
                         response_name = '2.16.840.1.113719.1.27.100.32'  # getBindDNResponse [NOVELL]
-                        response_value = encoder.encode(OctetString(self.bound))
+                        response_value = OctetString(self.bound)
                     elif extension[0] == '1.3.6.1.4.1.4203.1.11.3':  # WhoAmI [RFC4532]
                         result_code = 0
                         message = ''
                         response_name = '1.3.6.1.4.1.4203.1.11.3'  # WhoAmI [RFC4532]
-                        response_value = encoder.encode(OctetString(self.bound))
+                        response_value = OctetString(self.bound)
                     break
 
         return {'resultCode': result_code,
                 'matchedDN': '',
-                'diagnosticMessage': to_unicode(message, 'utf-8'),
+                'diagnosticMessage': to_unicode(message, SERVER_ENCODING),
                 'referral': None,
                 'responseName': response_name,
                 'responseValue': response_value
@@ -643,15 +745,13 @@ class MockBaseStrategy(object):
         if node.tag == ROOT:
             return node.elements[0].matched
         elif node.tag == AND:
-            for element in node.elements:
-                if not node.matched:
-                    node.matched.update(element.matched)
-                else:
-                    node.matched.intersection_update(element.matched)
-                if not node.unmatched:
-                    node.unmatched.update(element.unmatched)
-                else:
-                    node.unmatched.intersection_update(element.unmatched)
+            first_element = node.elements[0]
+            node.matched.update(first_element.matched)
+            node.unmatched.update(first_element.unmatched)
+
+            for element in node.elements[1:]:
+                node.matched.intersection_update(element.matched)
+                node.unmatched.intersection_update(element.unmatched)
         elif node.tag == OR:
             for element in node.elements:
                 node.matched.update(element.matched)
@@ -671,7 +771,7 @@ class MockBaseStrategy(object):
                             else:
                                 node.unmatched.add(candidate)
                         else:
-                            if to_unicode(value, 'utf-8').lower() >= to_unicode(attr_value, 'utf-8').lower():  # case insensitive string comparison
+                            if to_unicode(value, SERVER_ENCODING).lower() >= to_unicode(attr_value, SERVER_ENCODING).lower():  # case insensitive string comparison
                                 node.matched.add(candidate)
                             else:
                                 node.unmatched.add(candidate)
@@ -687,7 +787,7 @@ class MockBaseStrategy(object):
                             else:
                                 node.unmatched.add(candidate)
                         else:
-                            if to_unicode(value, 'utf-8').lower() <= to_unicode(attr_value, 'utf-8').lower():  # case insentive string comparison
+                            if to_unicode(value, SERVER_ENCODING).lower() <= to_unicode(attr_value, SERVER_ENCODING).lower():  # case insentive string comparison
                                 node.matched.add(candidate)
                             else:
                                 node.unmatched.add(candidate)
@@ -706,26 +806,26 @@ class MockBaseStrategy(object):
         elif node.tag == MATCH_SUBSTRING:
             attr_name = node.assertion['attr']
             # rebuild the original substring filter
-            if node.assertion['initial']:
-                substring_filter = re.escape(to_unicode(node.assertion['initial'], 'utf-8'))
+            if 'initial' in node.assertion and node.assertion['initial'] is not None:
+                substring_filter = re.escape(to_unicode(node.assertion['initial'], SERVER_ENCODING))
             else:
                 substring_filter = ''
 
-            if node.assertion['any']:
+            if 'any' in node.assertion and node.assertion['any'] is not None:
                 for middle in node.assertion['any']:
-                    substring_filter += '.*' + re.escape(to_unicode(middle, 'utf-8'))
+                    substring_filter += '.*' + re.escape(to_unicode(middle, SERVER_ENCODING))
 
-            if node.assertion['final']:
-                substring_filter += '.*' + re.escape(to_unicode(node.assertion['final'], 'utf-8'))
+            if 'final' in node.assertion and node.assertion['final'] is not None:
+                substring_filter += '.*' + re.escape(to_unicode(node.assertion['final'], SERVER_ENCODING))
 
-            if substring_filter and not node.assertion['any'] and not node.assertion['final']:  # only initial, adds .*
+            if substring_filter and not node.assertion.get('any', None) and not node.assertion.get('final', None):  # only initial, adds .*
                 substring_filter += '.*'
 
             regex_filter = re.compile(substring_filter, flags=re.UNICODE | re.IGNORECASE)  # unicode AND ignorecase
             for candidate in candidates:
                 if attr_name in self.connection.server.dit[candidate]:
                     for value in self.connection.server.dit[candidate][attr_name]:
-                        if regex_filter.match(to_unicode(value, 'utf-8')):
+                        if regex_filter.match(to_unicode(value, SERVER_ENCODING)):
                             node.matched.add(candidate)
                         else:
                             node.unmatched.add(candidate)
@@ -742,8 +842,8 @@ class MockBaseStrategy(object):
                 #     formatted_values = format_attribute_values(self.connection.server.schema, attr_name, self.connection.server.dit[candidate][attr_name], None)
                 #     if not isinstance(formatted_values, SEQUENCE_TYPES):
                 #         formatted_values = [formatted_values]
-                #     # if attr_value.decode('utf-8') in formatted_values:  # attributes values should be returned in utf-8
-                #     if self.equal(attr_name, attr_value.decode('utf-8'), formatted_values):  # attributes values should be returned in utf-8
+                #     # if attr_value.decode(SERVER_ENCODING) in formatted_values:  # attributes values should be returned in utf-8
+                #     if self.equal(attr_name, attr_value.decode(SERVER_ENCODING), formatted_values):  # attributes values should be returned in utf-8
                 #         node.matched.add(candidate)
                 #     else:
                 #         node.unmatched.add(candidate)
@@ -771,13 +871,34 @@ class MockBaseStrategy(object):
 
     @staticmethod
     def _check_equality(value1, value2):
-        if value1.isdigit() and value2.isdigit():
+        if str(value1).isdigit() and str(value2).isdigit():
             if int(value1) == int(value2):  # int comparison
                 return True
         try:
-            if to_unicode(value1, 'utf-8').lower() == to_unicode(value2, 'utf-8').lower():  # case insensitive comparison
+            if to_unicode(value1, SERVER_ENCODING).lower() == to_unicode(value2, SERVER_ENCODING).lower():  # case insensitive comparison
                 return True
-        except UnicodeDecodeError:
+        except UnicodeError:
             pass
 
         return False
+
+    def send(self, message_type, request, controls=None):
+        self.connection.request = self.decode_request(message_type, request, controls)
+        if self.connection.listening:
+            message_id = self.connection.server.next_message_id()
+            if self.connection.usage:  # ldap message is built for updating metrics only
+                ldap_message = LDAPMessage()
+                ldap_message['messageID'] = MessageID(message_id)
+                ldap_message['protocolOp'] = ProtocolOp().setComponentByName(message_type, request)
+                message_controls = build_controls_list(controls)
+                if message_controls is not None:
+                    ldap_message['controls'] = message_controls
+                asn1_request = BaseStrategy.decode_request(message_type, request, controls)
+                self.connection._usage.update_transmitted_message(asn1_request, len(encode(ldap_message)))
+            return message_id, message_type, request, controls
+        else:
+            self.connection.last_error = 'unable to send message, connection is not open'
+            if log_enabled(ERROR):
+                log(ERROR, '<%s> for <%s>', self.connection.last_error, self.connection)
+            raise LDAPSocketOpenError(self.connection.last_error)
+
