@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2014, 2015, 2016, 2017 Giovanni Cannata
+# Copyright 2014 - 2018 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -43,10 +43,10 @@ BOGUS_EXTENDED = -3
 BOGUS_ABANDON = -4
 
 try:
-    from queue import Queue
+    from queue import Queue, Empty
 except ImportError:  # Python 2
     # noinspection PyUnresolvedReferences
-    from Queue import Queue
+    from Queue import Queue, Empty
 
 
 # noinspection PyProtectedMember
@@ -89,7 +89,9 @@ class ReusableStrategy(BaseStrategy):
                 if not pool.started:  # if pool is not started remove it from the pools singleton and create a new onw
                     del ReusableStrategy.pools[connection.pool_name]
                     return object.__new__(cls)
-                if connection.pool_lifetime and pool.lifetime != connection.pool_lifetime:  # change lifetime
+                if connection.pool_keepalive and pool.keepalive != connection.pool_keepalive:  # change lifetime
+                    pool.keepalive = connection.pool_keepalive
+                if connection.pool_lifetime and pool.lifetime != connection.pool_lifetime:  # change keepalive
                     pool.lifetime = connection.pool_lifetime
                 if connection.pool_size and pool.pool_size != connection.pool_size:  # if pool size has changed terminate and recreate the connections
                     pool.terminate_pool()
@@ -105,6 +107,7 @@ class ReusableStrategy(BaseStrategy):
                 self.workers = []
                 self.pool_size = connection.pool_size or get_config_parameter('REUSABLE_THREADED_POOL_SIZE')
                 self.lifetime = connection.pool_lifetime or get_config_parameter('REUSABLE_THREADED_LIFETIME')
+                self.keepalive = connection.pool_keepalive
                 self.request_queue = Queue()
                 self.open_pool = False
                 self.bind_pool = False
@@ -113,7 +116,7 @@ class ReusableStrategy(BaseStrategy):
                 self.counter = 0
                 self.terminated_usage = ConnectionUsage() if connection._usage else None
                 self.terminated = False
-                self.lock = Lock()
+                self.pool_lock = Lock()
                 ReusableStrategy.pools[self.name] = self
                 self.started = False
                 if log_enabled(BASIC):
@@ -124,6 +127,7 @@ class ReusableStrategy(BaseStrategy):
             s += ' - responses in queue: ' + str(len(self._incoming))
             s += ' - pool size: ' + str(self.pool_size)
             s += ' - lifetime: ' + str(self.lifetime)
+            s += ' - keepalive: ' + str(self.keepalive)
             s += ' - open: ' + str(self.open_pool)
             s += ' - bind: ' + str(self.bind_pool)
             s += ' - tls: ' + str(self.tls_pool) + linesep
@@ -142,7 +146,7 @@ class ReusableStrategy(BaseStrategy):
 
         def get_info_from_server(self):
             for worker in self.workers:
-                with worker.lock:
+                with worker.worker_lock:
                     if not worker.connection.server.schema or not worker.connection.server.info:
                         worker.get_info_from_server = True
                     else:
@@ -150,7 +154,7 @@ class ReusableStrategy(BaseStrategy):
 
         def rebind_pool(self):
             for worker in self.workers:
-                with worker.lock:
+                with worker.worker_lock:
                     worker.connection.rebind(self.master_connection.user,
                                              self.master_connection.password,
                                              self.master_connection.authentication,
@@ -161,7 +165,7 @@ class ReusableStrategy(BaseStrategy):
             if not self.started:
                 self.create_pool()
                 for worker in self.workers:
-                    with worker.lock:
+                    with worker.worker_lock:
                         worker.thread.start()
                 self.started = True
                 self.terminated = False
@@ -207,8 +211,14 @@ class ReusableStrategy(BaseStrategy):
             terminate = False
             pool = self.master_connection.strategy.pool
             while not terminate:
-                counter, message_type, request, controls = pool.request_queue.get()
-                with self.worker.lock:
+                try:
+                    counter, message_type, request, controls = pool.request_queue.get(block=True, timeout=self.master_connection.strategy.pool.keepalive)
+                except Empty:  # issue an Abandon(0) operation to keep the connection live - Abandon(0) is a harmless operation
+                    if not self.worker.connection.closed:
+                        self.worker.connection.abandon(0)
+                    continue
+
+                with self.worker.worker_lock:
                     self.worker.busy = True
                     if counter == TERMINATE_REUSABLE:
                         terminate = True
@@ -252,7 +262,7 @@ class ReusableStrategy(BaseStrategy):
                                 result = self.worker.connection.result
                             except LDAPOperationResult as e:  # raise_exceptions has raised an exception. It must be redirected to the original connection thread
                                 exc = e
-                            with pool.lock:
+                            with pool.pool_lock:
                                 if exc:
                                     pool._incoming[counter] = (exc, None, None)
                                 else:
@@ -281,8 +291,8 @@ class ReusableStrategy(BaseStrategy):
             self.creation_time = None
             self.new_connection()
             self.task_counter = 0
-            self.thread = ReusableStrategy.PooledConnectionThread(self, connection)
-            self.lock = Lock()
+            self.thread = ReusableStrategy.PooledConnectionThread(self, self.master_connection)
+            self.worker_lock = Lock()
             if log_enabled(BASIC):
                 log(BASIC, 'instantiated PooledConnectionWorker: <%s>', self)
 
@@ -393,7 +403,7 @@ class ReusableStrategy(BaseStrategy):
                 self.pool.tls_pool = True
                 counter = BOGUS_EXTENDED
             else:
-                with self.pool.lock:
+                with self.pool.pool_lock:
                     self.pool.counter += 1
                     if self.pool.counter > LDAP_MAX_INT:
                         self.pool.counter = 1
@@ -441,7 +451,7 @@ class ReusableStrategy(BaseStrategy):
             result = None
             while timeout >= 0:  # waiting for completed message to appear in _incoming
                 try:
-                    with self.connection.strategy.pool.lock:
+                    with self.connection.strategy.pool.pool_lock:
                         response, result, request = self.connection.strategy.pool._incoming.pop(counter)
                 except KeyError:
                     sleep(sleeptime)
