@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2013 - 2018 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -25,9 +25,9 @@
 
 import socket
 
-from .. import SESSION_TERMINATED_BY_SERVER, RESPONSE_COMPLETE, SEQUENCE_TYPES, get_config_parameter
+from .. import SEQUENCE_TYPES, get_config_parameter
 from ..core.exceptions import LDAPSocketReceiveError, communication_exception_factory, LDAPExceptionError, LDAPExtensionError, LDAPOperationResult
-from ..strategy.base import BaseStrategy
+from ..strategy.base import BaseStrategy, SESSION_TERMINATED_BY_SERVER, RESPONSE_COMPLETE, TRANSACTION_ERROR
 from ..protocol.rfc4511 import LDAPMessage
 from ..utils.log import log, log_enabled, ERROR, NETWORK, EXTENDED, format_ldap_message
 from ..utils.asn1 import decoder, decode_message_fast
@@ -50,6 +50,7 @@ class SyncStrategy(BaseStrategy):
         self.no_real_dsa = False
         self.pooled = False
         self.can_stream = False
+        self.socket_size = get_config_parameter('SOCKET_SIZE')
 
     def open(self, reset_usage=True, read_server_info=True):
         BaseStrategy.open(self, reset_usage, read_server_info)
@@ -75,11 +76,10 @@ class SyncStrategy(BaseStrategy):
         data = b''
         get_more_data = True
         exc = None
-        socket_size = get_config_parameter('SOCKET_SIZE')
         while receiving:
             if get_more_data:
                 try:
-                    data = self.connection.socket.recv(socket_size)
+                    data = self.connection.socket.recv(self.socket_size)
                 except (OSError, socket.error, AttributeError) as e:
                     self.connection.last_error = 'error receiving data: ' + str(e)
                     exc = e
@@ -178,6 +178,8 @@ class SyncStrategy(BaseStrategy):
                         elif int(ldap_resp['messageID']) == 0:  # 0 is reserved for 'Unsolicited Notification' from server as per RFC4511 (paragraph 4.4)
                             if dict_response['responseName'] == '1.3.6.1.4.1.1466.20036':  # Notice of Disconnection as per RFC4511 (paragraph 4.4.1)
                                 return SESSION_TERMINATED_BY_SERVER
+                            elif dict_response['responseName'] == '2.16.840.1.113719.1.27.103.4':  # Novell LDAP transaction error unsolicited notification
+                                return TRANSACTION_ERROR
                             else:
                                 self.connection.last_error = 'unknown unsolicited notification from server'
                                 if log_enabled(ERROR):
