@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2015 - 2018 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -23,13 +23,11 @@
 # along with ldap3 in the COPYING and COPYING.LESSER files.
 # If not, see <http://www.gnu.org/licenses/>.
 
-
-from pyasn1.codec.ber.encoder import tagMap, BooleanEncoder
-from pyasn1.type.univ import Boolean
-from pyasn1.compat.octets import ints2octs
-
-from .. import RESULT_CODES
-
+from pyasn1 import __version__ as pyasn1_version
+from pyasn1.codec.ber import decoder  # for usage in other modules
+from pyasn1.codec.ber.encoder import Encoder # for monkeypatching of boolean value
+from ..core.results import RESULT_CODES
+from ..utils.conv import to_unicode
 from ..protocol.convert import referrals_to_list
 
 CLASSES = {(False, False): 0,  # Universal
@@ -40,16 +38,49 @@ CLASSES = {(False, False): 0,  # Universal
 
 # Monkeypatching of pyasn1 for encoding Boolean with the value 0xFF for TRUE
 # THIS IS NOT PART OF THE FAST BER DECODER
-class BooleanCEREncoder(BooleanEncoder):
-    _true = ints2octs((255,))
+if pyasn1_version == 'xxx0.2.3':
+    from pyasn1.codec.ber.encoder import tagMap, BooleanEncoder, encode
+    from pyasn1.type.univ import Boolean
+    from pyasn1.compat.octets import ints2octs
+    class BooleanCEREncoder(BooleanEncoder):
+        _true = ints2octs((255,))
 
-tagMap[Boolean.tagSet] = BooleanCEREncoder()
+    tagMap[Boolean.tagSet] = BooleanCEREncoder()
+else:
+    from pyasn1.codec.ber.encoder import tagMap, typeMap, AbstractItemEncoder
+    from pyasn1.type.univ import Boolean
+    from copy import deepcopy
 
-from pyasn1.codec.ber import encoder, decoder  # for usage in other modules
+    class LDAPBooleanEncoder(AbstractItemEncoder):
+        supportIndefLenMode = False
+        if pyasn1_version <= '0.2.3':
+            from pyasn1.compat.octets import ints2octs
+            _true = ints2octs((255,))
+            _false = ints2octs((0,))
+            def encodeValue(self, encodeFun, value, defMode, maxChunkSize):
+                return value and self._true or self._false, 0
+        elif pyasn1_version <= '0.3.1':
+            def encodeValue(self, encodeFun, value, defMode, maxChunkSize):
+                return value and (255,) or (0,), False, False
+        elif pyasn1_version <= '0.3.4':
+            def encodeValue(self, encodeFun, value, defMode, maxChunkSize, ifNotEmpty=False):
+                return value and (255,) or (0,), False, False
+        elif pyasn1_version <= '0.3.7':
+            def encodeValue(self, value, encodeFun, **options):
+                return value and (255,) or (0,), False, False
+        else:
+            def encodeValue(self, value, asn1Spec, encodeFun, **options):
+                return value and (255,) or (0,), False, False
+
+    customTagMap = deepcopy(tagMap)
+    customTypeMap = deepcopy(typeMap)
+    customTagMap[Boolean.tagSet] = LDAPBooleanEncoder()
+    customTypeMap[Boolean.typeId] = LDAPBooleanEncoder()
+
+    encode = Encoder(customTagMap, customTypeMap)
+# end of monkey patching
 
 # a fast BER decoder for LDAP responses only
-
-
 def compute_ber_size(data):
     """
     Compute size according to BER definite length rules
@@ -142,10 +173,10 @@ def ldap_result_to_dict_fast(response):
     response_dict = dict()
     response_dict['result'] = int(response[0][3])  # resultCode
     response_dict['description'] = RESULT_CODES[response_dict['result']]
-    response_dict['dn'] = response[1][3].decode('utf-8')  # matchedDN
-    response_dict['message'] = response[2][3].decode('utf-8')  # diagnosticMessage
+    response_dict['dn'] = to_unicode(response[1][3], from_server=True)  # matchedDN
+    response_dict['message'] = to_unicode(response[2][3], from_server=True)  # diagnosticMessage
     if len(response) == 4:
-        response_dict['referrals'] = referrals_to_list([referral[3].decode('utf-8') for referral in response[3][3]])  # referrals
+        response_dict['referrals'] = referrals_to_list([to_unicode(referral[3], from_server=True) for referral in response[3][3]])  # referrals
     else:
         response_dict['referrals'] = None
 
@@ -154,13 +185,13 @@ def ldap_result_to_dict_fast(response):
 
 ######
 
-if str != bytes:  # python 3
+if str is not bytes:  # Python 3
     def get_byte(x):
         return x
 
     def get_bytes(x):
         return x
-else:  # python 2
+else:  # Python 2
     def get_byte(x):
         return ord(x)
 
