@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2014, 2015, 2016, 2017 Giovanni Cannata
+# Copyright 2014 - 2018 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -32,33 +32,41 @@ from ..utils.ciDict import CaseInsensitiveDict
 from ..core.exceptions import LDAPDefinitionError
 
 
-def to_unicode(obj, encoding=None, additional_encodings=False):
+def to_unicode(obj, encoding=None, from_server=False):
     """Try to convert bytes (and str in python2) to unicode.
      Return object unmodified if python3 string, else raise an exception
     """
-    conf_default_encoding = get_config_parameter('DEFAULT_ENCODING')
-    conf_additional_encodings = get_config_parameter('ADDITIONAL_ENCODINGS')
+    conf_default_client_encoding = get_config_parameter('DEFAULT_CLIENT_ENCODING')
+    conf_default_server_encoding = get_config_parameter('DEFAULT_SERVER_ENCODING')
+    conf_additional_server_encodings = get_config_parameter('ADDITIONAL_SERVER_ENCODINGS')
     if isinstance(obj, NUMERIC_TYPES):
         obj = str(obj)
 
     if isinstance(obj, (bytes, bytearray)):
-        if encoding is None:
-            encoding = conf_default_encoding
-        try:
-            return obj.decode(encoding)
-        except UnicodeDecodeError:
-            if additional_encodings:
-                for encoding in conf_additional_encodings:  # AD could have DN not encoded in utf-8 (even if this is not allowed by RFC4510)
+        if from_server:  # data from server
+            if encoding is None:
+                encoding = conf_default_server_encoding
+            try:
+                return obj.decode(encoding)
+            except UnicodeDecodeError:
+                for encoding in conf_additional_server_encodings:  # AD could have DN not encoded in utf-8 (even if this is not allowed by RFC4510)
                     try:
                         return obj.decode(encoding)
                     except UnicodeDecodeError:
                         pass
+                raise UnicodeError("Unable to convert server data to unicode: %r" % obj)
+        else:  # data from client
+            if encoding is None:
+                encoding = conf_default_client_encoding
+            try:
+                return obj.decode(encoding)
+            except UnicodeDecodeError:
+                raise UnicodeError("Unable to convert client data to unicode: %r" % obj)
 
     if isinstance(obj, STRING_TYPES):  # python3 strings, python 2 unicode
         return obj
 
-    raise UnicodeError("Unable to convert to unicode %r" % obj)
-
+    raise UnicodeError("Unable to convert type %s to unicode: %r" % (type(obj).__class__.__name__, obj))
 
 def to_raw(obj, encoding='utf-8'):
     """Tries to convert to raw bytes from unicode"""
